﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// This class does not have a ParameterName, because its the root element of the ParameterConfiguration-Tree
  /// It only deals with child-ParameterConfigurations
  /// </summary>
  [StorableClass]
  public class RootParameterConfiguration : ParameterConfiguration {

    public new IParameterizedNamedItem Value {
      get { return (IParameterizedNamedItem)base.Value; }
      set { base.Value = value; }
    }

    public RootParameterConfiguration(IParameterizedNamedItem rootItem) {
      this.OptimizationEnabled = true;
      this.Value = rootItem;
    }

    public RootParameterConfiguration() { }
    [StorableConstructor]
    protected RootParameterConfiguration(bool deserializing) { }
    protected RootParameterConfiguration(RootParameterConfiguration original, Cloner cloner) : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new RootParameterConfiguration(this, cloner);
    }

    public override IParameter Parameter {
      get { return null; }
    }

    #region INamedItem Properties
    public override string Name {
      get { return Value.Name; }
      set { throw new NotSupportedException(); }
    }
    public override string Description {
      get { return Value.Description; }
      set { throw new NotSupportedException(); }
    }
    public override bool CanChangeDescription {
      get { return false; }
    }
    public override bool CanChangeName {
      get { return false; }
    }
    public override string ItemDescription {
      get { return Value.ItemDescription; }
    }
    public override System.Drawing.Image ItemImage {
      get { return Value.ItemImage; }
    }
    public override string ItemName {
      get { return Value.ItemName; }
    }
    public override Version ItemVersion {
      get { return Value.ItemVersion; }
    }
    #endregion
  }
}
