﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Core;
using HeuristicLab.Collections;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public abstract class ValueTypeParameterConfiguration<T> : ParameterConfiguration where T : class, IItem, IStringConvertibleValue, IDeepCloneable {
    [Storable]
    private IItemList<IRange<T>> ranges;
    public IItemList<IRange<T>> Ranges {
      get { return ranges; }
    }

    public ValueTypeParameterConfiguration(ParameterConfiguration parentConfiguration, string parameterName, IItem value) : base(parentConfiguration, parameterName, value) {
      this.ranges = new ItemList<IRange<T>>();
      this.ranges.ItemsAdded += new CollectionItemsChangedEventHandler<IndexedItem<IRange<T>>>(ranges_ItemsAdded);
      this.ranges.ItemsRemoved += new CollectionItemsChangedEventHandler<IndexedItem<IRange<T>>>(ranges_ItemsRemoved);
      this.ranges.CollectionReset += new CollectionItemsChangedEventHandler<IndexedItem<IRange<T>>>(ranges_CollectionReset);
    }

    protected ValueTypeParameterConfiguration() { }
    [StorableConstructor]
    protected ValueTypeParameterConfiguration(bool deserializing) : base(deserializing) { }
    protected ValueTypeParameterConfiguration(ValueTypeParameterConfiguration<T> original, Cloner cloner) : base(original, cloner) {
      this.ranges = cloner.Clone(original.Ranges);
    }

    void ranges_CollectionReset(object sender, CollectionItemsChangedEventArgs<IndexedItem<IRange<T>>> e) {
      foreach (var i in e.Items) {
        DeregisterItemEvents(i.Value);
      }
      OnStringChanged(sender, e);
    }

    void ranges_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<IndexedItem<IRange<T>>> e) {
      foreach (var i in e.Items) {
        DeregisterItemEvents(i.Value);
      }
      OnStringChanged(sender, e);
    }

    void ranges_ItemsAdded(object sender, CollectionItemsChangedEventArgs<IndexedItem<IRange<T>>> e) {
      foreach (var i in e.Items) {
        RegisterItemEvents(i.Value);
      }
      OnStringChanged(sender, e);
    }

    private void DeregisterItemEvents(IRange<T> range) {
      range.ToStringChanged -= new EventHandler(range_ToStringChanged);
    }
    private void RegisterItemEvents(IRange<T> range) {
      range.ToStringChanged += new EventHandler(range_ToStringChanged);
    }

    public IntValue GetRandomValue() {
      return new IntValue(0);
    }
    
    void range_ToStringChanged(object sender, EventArgs e) {
      OnStringChanged(sender, e);
    }

    public override string ToString() {
      return string.Format("{0}: ({1})", base.ToString(), string.Join(",", ranges.Select(r => r.ToString()).ToArray()));
    }

    //public override void SetRandomValue(IRandom random) {
    //  int rangeIndex = random.Next(ranges.Count);
    //  int value = random.Next(ranges.ElementAt(rangeIndex).LowerBound.Value, ranges.ElementAt(rangeIndex).UpperBound.Value); // TODO: respect StepSize parameter
    //  this.Parameter.ActualValue = new IntValue(value);
    //}
  }
}
